@@ -4,50 +4,49 @@ import logging
 from random import randrange
 
 from pyatv import connect, exceptions, scan
-from pyatv.const import Protocol
+from pyatv.const import DeviceModel, Protocol
+from pyatv.convert import model_str
 
-from homeassistant.components.media_player import DOMAIN as MP_DOMAIN
-from homeassistant.components.remote import DOMAIN as REMOTE_DOMAIN
+from homeassistant.components import zeroconf
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
+    ATTR_CONNECTIONS,
+    ATTR_IDENTIFIERS,
+    ATTR_MANUFACTURER,
+    ATTR_MODEL,
+    ATTR_NAME,
+    ATTR_SUGGESTED_AREA,
+    ATTR_SW_VERSION,
     CONF_ADDRESS,
     CONF_NAME,
-    CONF_PROTOCOL,
     EVENT_HOMEASSISTANT_STOP,
+    Platform,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import device_registry as dr
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
 )
-from homeassistant.helpers.entity import Entity
+from homeassistant.helpers.entity import DeviceInfo, Entity
 
-from .const import CONF_CREDENTIALS, CONF_IDENTIFIER, CONF_START_OFF, DOMAIN
+from .const import CONF_CREDENTIALS, CONF_IDENTIFIERS, CONF_START_OFF, DOMAIN
 
 _LOGGER = logging.getLogger(__name__)
 
 DEFAULT_NAME = "Apple TV"
 
+BACKOFF_TIME_LOWER_LIMIT = 15  # seconds
 BACKOFF_TIME_UPPER_LIMIT = 300  # Five minutes
 
-NOTIFICATION_TITLE = "Apple TV Notification"
-NOTIFICATION_ID = "apple_tv_notification"
-
-SOURCE_REAUTH = "reauth"
-
 SIGNAL_CONNECTED = "apple_tv_connected"
 SIGNAL_DISCONNECTED = "apple_tv_disconnected"
 
-PLATFORMS = [MP_DOMAIN, REMOTE_DOMAIN]
+PLATFORMS = [Platform.MEDIA_PLAYER, Platform.REMOTE]
 
 
-async def async_setup(hass, config):
-    """Set up the Apple TV integration."""
-    return True
-
-
-async def async_setup_entry(hass, entry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up a config entry for Apple TV."""
     manager = AppleTVManager(hass, entry)
     hass.data.setdefault(DOMAIN, {})[entry.unique_id] = manager
@@ -56,15 +55,17 @@ async def async_setup_entry(hass, entry):
         """Stop push updates when hass stops."""
         await manager.disconnect()
 
-    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, on_hass_stop)
+    entry.async_on_unload(
+        hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, on_hass_stop)
+    )
 
     async def setup_platforms():
         """Set up platforms and initiate connection."""
         await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_setup(entry, component)
-                for component in PLATFORMS
-            ]
+            *(
+                hass.config_entries.async_forward_entry_setup(entry, platform)
+                for platform in PLATFORMS
+            )
         )
         await manager.init()
 
@@ -73,16 +74,10 @@ async def async_setup_entry(hass, entry):
     return True
 
 
-async def async_unload_entry(hass, entry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload an Apple TV config entry."""
-    unload_ok = all(
-        await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_unload(entry, platform)
-                for platform in PLATFORMS
-            ]
-        )
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
+
     if unload_ok:
         manager = hass.data[DOMAIN].pop(entry.unique_id)
         await manager.disconnect()
@@ -93,12 +88,15 @@ async def async_unload_entry(hass, entry):
 class AppleTVEntity(Entity):
     """Device that sends commands to an Apple TV."""
 
+    _attr_should_poll = False
+
     def __init__(self, name, identifier, manager):
         """Initialize device."""
         self.atv = None
         self.manager = manager
-        self._name = name
-        self._identifier = identifier
+        self._attr_name = name
+        self._attr_unique_id = identifier
+        self._attr_device_info = DeviceInfo(identifiers={(DOMAIN, identifier)})
 
     async def async_added_to_hass(self):
         """Handle when an entity is about to be added to Home Assistant."""
@@ -119,13 +117,13 @@ class AppleTVEntity(Entity):
 
         self.async_on_remove(
             async_dispatcher_connect(
-                self.hass, f"{SIGNAL_CONNECTED}_{self._identifier}", _async_connected
+                self.hass, f"{SIGNAL_CONNECTED}_{self.unique_id}", _async_connected
             )
         )
         self.async_on_remove(
             async_dispatcher_connect(
                 self.hass,
-                f"{SIGNAL_DISCONNECTED}_{self._identifier}",
+                f"{SIGNAL_DISCONNECTED}_{self.unique_id}",
                 _async_disconnected,
             )
         )
@@ -136,21 +134,6 @@ class AppleTVEntity(Entity):
     def async_device_disconnected(self):
         """Handle when connection was lost to device."""
 
-    @property
-    def name(self):
-        """Return the name of the device."""
-        return self._name
-
-    @property
-    def unique_id(self):
-        """Return a unique ID."""
-        return self._identifier
-
-    @property
-    def should_poll(self):
-        """No polling needed for Apple TV."""
-        return False
-
 
 class AppleTVManager:
     """Connection and power manager for an Apple TV.
@@ -181,7 +164,7 @@ class AppleTVManager:
         This is a callback function from pyatv.interface.DeviceListener.
         """
         _LOGGER.warning(
-            'Connection lost to Apple TV "%s"', self.config_entry.data.get(CONF_NAME)
+            'Connection lost to Apple TV "%s"', self.config_entry.data[CONF_NAME]
         )
         self._connection_was_lost = True
         self._handle_disconnect()
@@ -196,7 +179,6 @@ class AppleTVManager:
     def _handle_disconnect(self):
         """Handle that the device disconnected and restart connect loop."""
         if self.atv:
-            self.atv.listener = None
             self.atv.close()
             self.atv = None
         self._dispatch_send(SIGNAL_DISCONNECTED)
@@ -213,8 +195,6 @@ class AppleTVManager:
         self._is_on = False
         try:
             if self.atv:
-                self.atv.push_updater.listener = None
-                self.atv.push_updater.stop()
                 self.atv.close()
                 self.atv = None
             if self._task:
@@ -244,7 +224,12 @@ class AppleTVManager:
                 if conf:
                     await self._connect(conf)
             except exceptions.AuthenticationError:
-                self._auth_problem()
+                self.config_entry.async_start_reauth(self.hass)
+                asyncio.create_task(self.disconnect())
+                _LOGGER.exception(
+                    "Authentication failed for %s, try reconfiguring device",
+                    self.config_entry.data[CONF_NAME],
+                )
                 break
             except asyncio.CancelledError:
                 pass
@@ -255,7 +240,11 @@ class AppleTVManager:
             if self.atv is None:
                 self._connection_attempts += 1
                 backoff = min(
-                    randrange(2 ** self._connection_attempts), BACKOFF_TIME_UPPER_LIMIT
+                    max(
+                        BACKOFF_TIME_LOWER_LIMIT,
+                        randrange(2 ** self._connection_attempts),
+                    ),
+                    BACKOFF_TIME_UPPER_LIMIT,
                 )
 
                 _LOGGER.debug("Reconnecting in %d seconds", backoff)
@@ -264,57 +253,38 @@ class AppleTVManager:
         _LOGGER.debug("Connect loop ended")
         self._task = None
 
-    def _auth_problem(self):
-        """Problem to authenticate occurred that needs intervention."""
-        _LOGGER.debug("Authentication error, reconfigure integration")
-
-        name = self.config_entry.data.get(CONF_NAME)
-        identifier = self.config_entry.unique_id
-
-        self.hass.components.persistent_notification.create(
-            "An irrecoverable connection problem occurred when connecting to "
-            f"`f{name}`. Please go to the Integrations page and reconfigure it",
-            title=NOTIFICATION_TITLE,
-            notification_id=NOTIFICATION_ID,
-        )
-
-        # Add to event queue as this function is called from a task being
-        # cancelled from disconnect
-        asyncio.create_task(self.disconnect())
-
-        self.hass.async_create_task(
-            self.hass.config_entries.flow.async_init(
-                DOMAIN,
-                context={"source": SOURCE_REAUTH},
-                data={CONF_NAME: name, CONF_IDENTIFIER: identifier},
-            )
-        )
-
     async def _scan(self):
         """Try to find device by scanning for it."""
-        identifier = self.config_entry.unique_id
+        identifiers = set(
+            self.config_entry.data.get(CONF_IDENTIFIERS, [self.config_entry.unique_id])
+        )
         address = self.config_entry.data[CONF_ADDRESS]
-        protocol = Protocol(self.config_entry.data[CONF_PROTOCOL])
 
-        _LOGGER.debug("Discovering device %s", identifier)
+        # Only scan for and set up protocols that was successfully paired
+        protocols = {
+            Protocol(int(protocol))
+            for protocol in self.config_entry.data[CONF_CREDENTIALS]
+        }
+
+        _LOGGER.debug("Discovering device %s", self.config_entry.title)
+        aiozc = await zeroconf.async_get_async_instance(self.hass)
         atvs = await scan(
-            self.hass.loop, identifier=identifier, protocol=protocol, hosts=[address]
+            self.hass.loop,
+            identifier=identifiers,
+            protocol=protocols,
+            hosts=[address],
+            aiozc=aiozc,
         )
         if atvs:
             return atvs[0]
 
         _LOGGER.debug(
-            "Failed to find device %s with address %s, trying to scan",
-            identifier,
+            "Failed to find device %s with address %s",
+            self.config_entry.title,
             address,
         )
-
-        atvs = await scan(self.hass.loop, identifier=identifier, protocol=protocol)
-        if atvs:
-            return atvs[0]
-
-        _LOGGER.debug("Failed to find device %s, trying later", identifier)
-
+        # We no longer multicast scan for the device since as soon as async_step_zeroconf runs,
+        # it will update the address and reload the config entry when the device is found.
         return None
 
     async def _connect(self, conf):
@@ -322,8 +292,16 @@ class AppleTVManager:
         credentials = self.config_entry.data[CONF_CREDENTIALS]
         session = async_get_clientsession(self.hass)
 
-        for protocol, creds in credentials.items():
-            conf.set_credentials(Protocol(int(protocol)), creds)
+        for protocol_int, creds in credentials.items():
+            protocol = Protocol(int(protocol_int))
+            if conf.get_service(protocol) is not None:
+                conf.set_credentials(protocol, creds)
+            else:
+                _LOGGER.warning(
+                    "Protocol %s not found for %s, functionality will be reduced",
+                    protocol.name,
+                    self.config_entry.data[CONF_NAME],
+                )
 
         _LOGGER.debug("Connecting to device %s", self.config_entry.data[CONF_NAME])
         self.atv = await connect(conf, self.hass.loop, session=session)
@@ -332,32 +310,44 @@ class AppleTVManager:
         self._dispatch_send(SIGNAL_CONNECTED, self.atv)
         self._address_updated(str(conf.address))
 
-        await self._async_setup_device_registry()
+        self._async_setup_device_registry()
 
         self._connection_attempts = 0
         if self._connection_was_lost:
             _LOGGER.info(
-                'Connection was re-established to Apple TV "%s"', self.atv.service.name
+                'Connection was re-established to device "%s"',
+                self.config_entry.data[CONF_NAME],
             )
             self._connection_was_lost = False
 
-    async def _async_setup_device_registry(self):
+    @callback
+    def _async_setup_device_registry(self):
         attrs = {
-            "identifiers": {(DOMAIN, self.config_entry.unique_id)},
-            "manufacturer": "Apple",
-            "name": self.config_entry.data[CONF_NAME],
+            ATTR_IDENTIFIERS: {(DOMAIN, self.config_entry.unique_id)},
+            ATTR_MANUFACTURER: "Apple",
+            ATTR_NAME: self.config_entry.data[CONF_NAME],
         }
 
+        area = attrs[ATTR_NAME]
+        name_trailer = f" {DEFAULT_NAME}"
+        if area.endswith(name_trailer):
+            area = area[: -len(name_trailer)]
+        attrs[ATTR_SUGGESTED_AREA] = area
+
         if self.atv:
             dev_info = self.atv.device_info
 
-            attrs["model"] = "Apple TV " + dev_info.model.name.replace("Gen", "")
-            attrs["sw_version"] = dev_info.version
+            attrs[ATTR_MODEL] = (
+                dev_info.raw_model
+                if dev_info.model == DeviceModel.Unknown and dev_info.raw_model
+                else model_str(dev_info.model)
+            )
+            attrs[ATTR_SW_VERSION] = dev_info.version
 
             if dev_info.mac:
-                attrs["connections"] = {(dr.CONNECTION_NETWORK_MAC, dev_info.mac)}
+                attrs[ATTR_CONNECTIONS] = {(dr.CONNECTION_NETWORK_MAC, dev_info.mac)}
 
-        device_registry = await dr.async_get_registry(self.hass)
+        device_registry = dr.async_get(self.hass)
         device_registry.async_get_or_create(
             config_entry_id=self.config_entry.entry_id, **attrs
         )
