@@ -1,66 +1,52 @@
 """Support for Homekit motion sensors."""
-import logging
-
 from aiohomekit.model.characteristics import CharacteristicsTypes
+from aiohomekit.model.services import ServicesTypes
 
 from homeassistant.components.binary_sensor import (
-    DEVICE_CLASS_MOISTURE,
-    DEVICE_CLASS_MOTION,
-    DEVICE_CLASS_OCCUPANCY,
-    DEVICE_CLASS_OPENING,
-    DEVICE_CLASS_SMOKE,
-    BinarySensorDevice,
+    BinarySensorDeviceClass,
+    BinarySensorEntity,
 )
-from homeassistant.core import callback
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import KNOWN_DEVICES, HomeKitEntity
 
-_LOGGER = logging.getLogger(__name__)
 
-
-class HomeKitMotionSensor(HomeKitEntity, BinarySensorDevice):
+class HomeKitMotionSensor(HomeKitEntity, BinarySensorEntity):
     """Representation of a Homekit motion sensor."""
 
+    _attr_device_class = BinarySensorDeviceClass.MOTION
+
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
         return [CharacteristicsTypes.MOTION_DETECTED]
 
-    @property
-    def device_class(self):
-        """Define this binary_sensor as a motion sensor."""
-        return DEVICE_CLASS_MOTION
-
     @property
     def is_on(self):
         """Has motion been detected."""
         return self.service.value(CharacteristicsTypes.MOTION_DETECTED)
 
 
-class HomeKitContactSensor(HomeKitEntity, BinarySensorDevice):
+class HomeKitContactSensor(HomeKitEntity, BinarySensorEntity):
     """Representation of a Homekit contact sensor."""
 
+    _attr_device_class = BinarySensorDeviceClass.OPENING
+
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
         return [CharacteristicsTypes.CONTACT_STATE]
 
-    @property
-    def device_class(self):
-        """Define this binary_sensor as a opening sensor."""
-        return DEVICE_CLASS_OPENING
-
     @property
     def is_on(self):
         """Return true if the binary sensor is on/open."""
         return self.service.value(CharacteristicsTypes.CONTACT_STATE) == 1
 
 
-class HomeKitSmokeSensor(HomeKitEntity, BinarySensorDevice):
+class HomeKitSmokeSensor(HomeKitEntity, BinarySensorEntity):
     """Representation of a Homekit smoke sensor."""
 
-    @property
-    def device_class(self) -> str:
-        """Return the class of this sensor."""
-        return DEVICE_CLASS_SMOKE
+    _attr_device_class = BinarySensorDeviceClass.SMOKE
 
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
@@ -72,13 +58,25 @@ class HomeKitSmokeSensor(HomeKitEntity, BinarySensorDevice):
         return self.service.value(CharacteristicsTypes.SMOKE_DETECTED) == 1
 
 
-class HomeKitOccupancySensor(HomeKitEntity, BinarySensorDevice):
-    """Representation of a Homekit occupancy sensor."""
+class HomeKitCarbonMonoxideSensor(HomeKitEntity, BinarySensorEntity):
+    """Representation of a Homekit BO sensor."""
+
+    _attr_device_class = BinarySensorDeviceClass.GAS
+
+    def get_characteristic_types(self):
+        """Define the homekit characteristics the entity is tracking."""
+        return [CharacteristicsTypes.CARBON_MONOXIDE_DETECTED]
 
     @property
-    def device_class(self) -> str:
-        """Return the class of this sensor."""
-        return DEVICE_CLASS_OCCUPANCY
+    def is_on(self):
+        """Return true if CO is currently detected."""
+        return self.service.value(CharacteristicsTypes.CARBON_MONOXIDE_DETECTED) == 1
+
+
+class HomeKitOccupancySensor(HomeKitEntity, BinarySensorEntity):
+    """Representation of a Homekit occupancy sensor."""
+
+    _attr_device_class = BinarySensorDeviceClass.OCCUPANCY
 
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
@@ -90,18 +88,15 @@ class HomeKitOccupancySensor(HomeKitEntity, BinarySensorDevice):
         return self.service.value(CharacteristicsTypes.OCCUPANCY_DETECTED) == 1
 
 
-class HomeKitLeakSensor(HomeKitEntity, BinarySensorDevice):
+class HomeKitLeakSensor(HomeKitEntity, BinarySensorEntity):
     """Representation of a Homekit leak sensor."""
 
+    _attr_device_class = BinarySensorDeviceClass.MOISTURE
+
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
         return [CharacteristicsTypes.LEAK_DETECTED]
 
-    @property
-    def device_class(self):
-        """Define this binary_sensor as a leak sensor."""
-        return DEVICE_CLASS_MOISTURE
-
     @property
     def is_on(self):
         """Return true if a leak is detected from the binary sensor."""
@@ -109,25 +104,29 @@ class HomeKitLeakSensor(HomeKitEntity, BinarySensorDevice):
 
 
 ENTITY_TYPES = {
-    "motion": HomeKitMotionSensor,
-    "contact": HomeKitContactSensor,
-    "smoke": HomeKitSmokeSensor,
-    "occupancy": HomeKitOccupancySensor,
-    "leak": HomeKitLeakSensor,
+    ServicesTypes.MOTION_SENSOR: HomeKitMotionSensor,
+    ServicesTypes.CONTACT_SENSOR: HomeKitContactSensor,
+    ServicesTypes.SMOKE_SENSOR: HomeKitSmokeSensor,
+    ServicesTypes.CARBON_MONOXIDE_SENSOR: HomeKitCarbonMonoxideSensor,
+    ServicesTypes.OCCUPANCY_SENSOR: HomeKitOccupancySensor,
+    ServicesTypes.LEAK_SENSOR: HomeKitLeakSensor,
 }
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up Homekit lighting."""
     hkid = config_entry.data["AccessoryPairingID"]
     conn = hass.data[KNOWN_DEVICES][hkid]
 
     @callback
-    def async_add_service(aid, service):
-        entity_class = ENTITY_TYPES.get(service["stype"])
-        if not entity_class:
+    def async_add_service(service):
+        if not (entity_class := ENTITY_TYPES.get(service.short_type)):
             return False
-        info = {"aid": aid, "iid": service["iid"]}
+        info = {"aid": service.accessory.aid, "iid": service.iid}
         async_add_entities([entity_class(conn, info)], True)
         return True
 
