@@ -10,21 +10,21 @@ from pylutron_caseta.smartbridge import Smartbridge
 import voluptuous as vol
 
 from homeassistant import config_entries
-from homeassistant.components.zeroconf import ATTR_HOSTNAME
+from homeassistant.components import zeroconf
 from homeassistant.const import CONF_HOST, CONF_NAME
 from homeassistant.core import callback
+from homeassistant.data_entry_flow import FlowResult
 
 from .const import (
-    ABORT_REASON_ALREADY_CONFIGURED,
     ABORT_REASON_CANNOT_CONNECT,
     BRIDGE_TIMEOUT,
     CONF_CA_CERTS,
     CONF_CERTFILE,
     CONF_KEYFILE,
+    DOMAIN,
     ERROR_CANNOT_CONNECT,
     STEP_IMPORT_FAILED,
 )
-from .const import DOMAIN  # pylint: disable=unused-import
 
 HOSTNAME = "hostname"
 
@@ -47,7 +47,6 @@ class LutronCasetaFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
     """Handle Lutron Caseta config flow."""
 
     VERSION = 1
-    CONNECTION_CLASS = config_entries.CONN_CLASS_LOCAL_POLL
 
     def __init__(self):
         """Initialize a Lutron Caseta flow."""
@@ -64,16 +63,18 @@ class LutronCasetaFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
 
         return self.async_show_form(step_id="user", data_schema=DATA_SCHEMA_USER)
 
-    async def async_step_zeroconf(self, discovery_info):
+    async def async_step_zeroconf(
+        self, discovery_info: zeroconf.ZeroconfServiceInfo
+    ) -> FlowResult:
         """Handle a flow initialized by zeroconf discovery."""
-        hostname = discovery_info[ATTR_HOSTNAME]
-        if hostname is None or not hostname.startswith("lutron-"):
+        hostname = discovery_info.hostname
+        if hostname is None or not hostname.lower().startswith("lutron-"):
             return self.async_abort(reason="not_lutron_device")
 
         self.lutron_id = hostname.split("-")[1].replace(".local.", "")
 
         await self.async_set_unique_id(self.lutron_id)
-        host = discovery_info[CONF_HOST]
+        host = discovery_info.host
         self._abort_if_unique_id_configured({CONF_HOST: host})
 
         self.data[CONF_HOST] = host
@@ -83,7 +84,9 @@ class LutronCasetaFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         }
         return await self.async_step_link()
 
-    async def async_step_homekit(self, discovery_info):
+    async def async_step_homekit(
+        self, discovery_info: zeroconf.ZeroconfServiceInfo
+    ) -> FlowResult:
         """Handle a flow initialized by homekit discovery."""
         return await self.async_step_zeroconf(discovery_info)
 
@@ -91,8 +94,7 @@ class LutronCasetaFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         """Handle pairing with the hub."""
         errors = {}
         # Abort if existing entry with matching host exists.
-        if self._async_data_host_is_already_configured():
-            return self.async_abort(reason=ABORT_REASON_ALREADY_CONFIGURED)
+        self._async_abort_entries_match({CONF_HOST: self.data[CONF_HOST]})
 
         self._configure_tls_assets()
 
@@ -141,7 +143,9 @@ class LutronCasetaFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
     def _write_tls_assets(self, assets):
         """Write the tls assets to disk."""
         for asset_key, conf_key in FILE_MAPPING.items():
-            with open(self.hass.config.path(self.data[conf_key]), "w") as file_handle:
+            with open(
+                self.hass.config.path(self.data[conf_key]), "w", encoding="utf8"
+            ) as file_handle:
                 file_handle.write(assets[asset_key])
 
     def _tls_assets_exist(self):
@@ -157,28 +161,17 @@ class LutronCasetaFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         for asset_key, conf_key in FILE_MAPPING.items():
             self.data[conf_key] = TLS_ASSET_TEMPLATE.format(self.bridge_id, asset_key)
 
-    @callback
-    def _async_data_host_is_already_configured(self):
-        """Check to see if the host is already configured."""
-        return any(
-            self.data[CONF_HOST] == entry.data[CONF_HOST]
-            for entry in self._async_current_entries()
-            if CONF_HOST in entry.data
-        )
-
     async def async_step_import(self, import_info):
         """Import a new Caseta bridge as a config entry.
 
         This flow is triggered by `async_setup`.
         """
-
         host = import_info[CONF_HOST]
         # Store the imported config for other steps in this flow to access.
         self.data[CONF_HOST] = host
 
         # Abort if existing entry with matching host exists.
-        if self._async_data_host_is_already_configured():
-            return self.async_abort(reason=ABORT_REASON_ALREADY_CONFIGURED)
+        self._async_abort_entries_match({CONF_HOST: self.data[CONF_HOST]})
 
         self.data[CONF_KEYFILE] = import_info[CONF_KEYFILE]
         self.data[CONF_CERTFILE] = import_info[CONF_CERTFILE]
@@ -213,7 +206,6 @@ class LutronCasetaFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
 
     async def async_validate_connectable_bridge_config(self):
         """Check if we can connect to the bridge with the current config."""
-
         bridge = None
 
         try:
