@@ -1,30 +1,35 @@
 """Helper to handle a set of topics to subscribe to."""
+from __future__ import annotations
+
 from collections import deque
+from collections.abc import Callable
 from functools import wraps
-import logging
 from typing import Any
 
-from homeassistant.helpers.typing import HomeAssistantType
+from homeassistant.core import HomeAssistant
 
 from .const import ATTR_DISCOVERY_PAYLOAD, ATTR_DISCOVERY_TOPIC
 from .models import MessageCallbackType
 
-_LOGGER = logging.getLogger(__name__)
-
 DATA_MQTT_DEBUG_INFO = "mqtt_debug_info"
 STORED_MESSAGES = 10
 
 
-def log_messages(hass: HomeAssistantType, entity_id: str) -> MessageCallbackType:
+def log_messages(
+    hass: HomeAssistant, entity_id: str
+) -> Callable[[MessageCallbackType], MessageCallbackType]:
     """Wrap an MQTT message callback to support message logging."""
 
     def _log_message(msg):
         """Log message."""
         debug_info = hass.data[DATA_MQTT_DEBUG_INFO]
-        messages = debug_info["entities"][entity_id]["topics"][msg.subscribed_topic]
-        messages.append(msg.payload)
+        messages = debug_info["entities"][entity_id]["subscriptions"][
+            msg.subscribed_topic
+        ]["messages"]
+        if msg not in messages:
+            messages.append(msg)
 
-    def _decorator(msg_callback: MessageCallbackType):
+    def _decorator(msg_callback: MessageCallbackType) -> MessageCallbackType:
         @wraps(msg_callback)
         def wrapper(msg: Any) -> None:
             """Log message."""
@@ -37,24 +42,36 @@ def log_messages(hass: HomeAssistantType, entity_id: str) -> MessageCallbackType
     return _decorator
 
 
-def add_topic(hass, message_callback, topic):
-    """Prepare debug data for topic."""
-    entity_id = getattr(message_callback, "__entity_id", None)
-    if entity_id:
+def add_subscription(hass, message_callback, subscription):
+    """Prepare debug data for subscription."""
+    if entity_id := getattr(message_callback, "__entity_id", None):
         debug_info = hass.data.setdefault(
             DATA_MQTT_DEBUG_INFO, {"entities": {}, "triggers": {}}
         )
         entity_info = debug_info["entities"].setdefault(
-            entity_id, {"topics": {}, "discovery_data": {}}
+            entity_id, {"subscriptions": {}, "discovery_data": {}}
         )
-        entity_info["topics"][topic] = deque([], STORED_MESSAGES)
+        if subscription not in entity_info["subscriptions"]:
+            entity_info["subscriptions"][subscription] = {
+                "count": 0,
+                "messages": deque([], STORED_MESSAGES),
+            }
+        entity_info["subscriptions"][subscription]["count"] += 1
 
 
-def remove_topic(hass, message_callback, topic):
-    """Remove debug data for topic."""
+def remove_subscription(hass, message_callback, subscription):
+    """Remove debug data for subscription if it exists."""
     entity_id = getattr(message_callback, "__entity_id", None)
     if entity_id and entity_id in hass.data[DATA_MQTT_DEBUG_INFO]["entities"]:
-        hass.data[DATA_MQTT_DEBUG_INFO]["entities"][entity_id]["topics"].pop(topic)
+        hass.data[DATA_MQTT_DEBUG_INFO]["entities"][entity_id]["subscriptions"][
+            subscription
+        ]["count"] -= 1
+        if not hass.data[DATA_MQTT_DEBUG_INFO]["entities"][entity_id]["subscriptions"][
+            subscription
+        ]["count"]:
+            hass.data[DATA_MQTT_DEBUG_INFO]["entities"][entity_id]["subscriptions"].pop(
+                subscription
+            )
 
 
 def add_entity_discovery_data(hass, discovery_data, entity_id):
@@ -63,7 +80,7 @@ def add_entity_discovery_data(hass, discovery_data, entity_id):
         DATA_MQTT_DEBUG_INFO, {"entities": {}, "triggers": {}}
     )
     entity_info = debug_info["entities"].setdefault(
-        entity_id, {"topics": {}, "discovery_data": {}}
+        entity_id, {"subscriptions": {}, "discovery_data": {}}
     )
     entity_info["discovery_data"] = discovery_data
 
@@ -107,7 +124,7 @@ async def info_for_device(hass, device_id):
     entity_registry = await hass.helpers.entity_registry.async_get_registry()
 
     entries = hass.helpers.entity_registry.async_entries_for_device(
-        entity_registry, device_id
+        entity_registry, device_id, include_disabled_entities=True
     )
     mqtt_debug_info = hass.data.setdefault(
         DATA_MQTT_DEBUG_INFO, {"entities": {}, "triggers": {}}
@@ -117,9 +134,21 @@ async def info_for_device(hass, device_id):
             continue
 
         entity_info = mqtt_debug_info["entities"][entry.entity_id]
-        topics = [
-            {"topic": topic, "messages": list(messages)}
-            for topic, messages in entity_info["topics"].items()
+        subscriptions = [
+            {
+                "topic": topic,
+                "messages": [
+                    {
+                        "payload": str(msg.payload),
+                        "qos": msg.qos,
+                        "retain": msg.retain,
+                        "time": msg.timestamp,
+                        "topic": msg.topic,
+                    }
+                    for msg in list(subscription["messages"])
+                ],
+            }
+            for topic, subscription in entity_info["subscriptions"].items()
         ]
         discovery_data = {
             "topic": entity_info["discovery_data"].get(ATTR_DISCOVERY_TOPIC, ""),
@@ -128,13 +157,13 @@ async def info_for_device(hass, device_id):
         mqtt_info["entities"].append(
             {
                 "entity_id": entry.entity_id,
-                "topics": topics,
+                "subscriptions": subscriptions,
                 "discovery_data": discovery_data,
             }
         )
 
     for trigger in mqtt_debug_info["triggers"].values():
-        if trigger["device_id"] != device_id:
+        if trigger["device_id"] != device_id or trigger["discovery_data"] is None:
             continue
 
         discovery_data = {
