@@ -1,14 +1,11 @@
 """Media Player component to integrate TVs exposing the Joint Space API."""
-from typing import Any, Dict, Optional
+from __future__ import annotations
 
 from haphilipsjs import ConnectionFailure
-import voluptuous as vol
 
-from homeassistant import config_entries
 from homeassistant.components.media_player import (
-    DEVICE_CLASS_TV,
-    PLATFORM_SCHEMA,
     BrowseMedia,
+    MediaPlayerDeviceClass,
     MediaPlayerEntity,
 )
 from homeassistant.components.media_player.const import (
@@ -34,20 +31,15 @@ from homeassistant.components.media_player.const import (
     SUPPORT_VOLUME_STEP,
 )
 from homeassistant.components.media_player.errors import BrowseError
-from homeassistant.const import (
-    CONF_API_VERSION,
-    CONF_HOST,
-    CONF_NAME,
-    STATE_OFF,
-    STATE_ON,
-)
-from homeassistant.core import callback
-import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.typing import HomeAssistantType
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import STATE_OFF, STATE_ON
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
 from . import LOGGER as _LOGGER, PhilipsTVDataUpdateCoordinator
-from .const import CONF_SYSTEM, DOMAIN
+from .const import DOMAIN
 
 SUPPORT_PHILIPS_JS = (
     SUPPORT_TURN_OFF
@@ -66,54 +58,22 @@ SUPPORT_PHILIPS_JS = (
 
 CONF_ON_ACTION = "turn_on_action"
 
-DEFAULT_API_VERSION = 1
-
-PLATFORM_SCHEMA = vol.All(
-    cv.deprecated(CONF_HOST),
-    cv.deprecated(CONF_NAME),
-    cv.deprecated(CONF_API_VERSION),
-    cv.deprecated(CONF_ON_ACTION),
-    PLATFORM_SCHEMA.extend(
-        {
-            vol.Required(CONF_HOST): cv.string,
-            vol.Remove(CONF_NAME): cv.string,
-            vol.Optional(CONF_API_VERSION, default=DEFAULT_API_VERSION): vol.Coerce(
-                int
-            ),
-            vol.Remove(CONF_ON_ACTION): cv.SCRIPT_SCHEMA,
-        }
-    ),
-)
-
 
 def _inverted(data):
     return {v: k for k, v in data.items()}
 
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
-    """Set up the Philips TV platform."""
-    hass.async_create_task(
-        hass.config_entries.flow.async_init(
-            DOMAIN,
-            context={"source": config_entries.SOURCE_IMPORT},
-            data=config,
-        )
-    )
-
-
 async def async_setup_entry(
-    hass: HomeAssistantType,
-    config_entry: config_entries.ConfigEntry,
-    async_add_entities,
-):
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the configuration entry."""
     coordinator = hass.data[DOMAIN][config_entry.entry_id]
     async_add_entities(
         [
             PhilipsTVMediaPlayer(
                 coordinator,
-                config_entry.data[CONF_SYSTEM],
-                config_entry.unique_id,
             )
         ]
     )
@@ -122,25 +82,36 @@ async def async_setup_entry(
 class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
     """Representation of a Philips TV exposing the JointSpace API."""
 
+    _coordinator: PhilipsTVDataUpdateCoordinator
+    _attr_device_class = MediaPlayerDeviceClass.TV
+
     def __init__(
         self,
         coordinator: PhilipsTVDataUpdateCoordinator,
-        system: Dict[str, Any],
-        unique_id: str,
-    ):
+    ) -> None:
         """Initialize the Philips TV."""
         self._tv = coordinator.api
         self._coordinator = coordinator
         self._sources = {}
         self._channels = {}
         self._supports = SUPPORT_PHILIPS_JS
-        self._system = system
-        self._unique_id = unique_id
+        self._system = coordinator.system
+        self._attr_name = coordinator.system["name"]
+        self._attr_unique_id = coordinator.unique_id
+        self._attr_device_info = DeviceInfo(
+            identifiers={
+                (DOMAIN, coordinator.unique_id),
+            },
+            manufacturer="Philips",
+            model=coordinator.system.get("model"),
+            sw_version=coordinator.system.get("softwareversion"),
+            name=coordinator.system["name"],
+        )
         self._state = STATE_OFF
-        self._media_content_type: Optional[str] = None
-        self._media_content_id: Optional[str] = None
-        self._media_title: Optional[str] = None
-        self._media_channel: Optional[str] = None
+        self._media_content_type: str | None = None
+        self._media_content_id: str | None = None
+        self._media_title: str | None = None
+        self._media_channel: str | None = None
 
         super().__init__(coordinator)
         self._update_from_coordinator()
@@ -150,11 +121,6 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
         self.async_write_ha_state()
         await self.coordinator.async_request_refresh()
 
-    @property
-    def name(self):
-        """Return the device name."""
-        return self._system["name"]
-
     @property
     def supported_features(self):
         """Flag media player features that are supported."""
@@ -168,9 +134,8 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
     @property
     def state(self):
         """Get the device state. An exception means OFF state."""
-        if self._tv.on:
-            if self._tv.powerstate == "On" or self._tv.powerstate is None:
-                return STATE_ON
+        if self._tv.on and (self._tv.powerstate == "On" or self._tv.powerstate is None):
+            return STATE_ON
         return STATE_OFF
 
     @property
@@ -185,8 +150,7 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
 
     async def async_select_source(self, source):
         """Set the input source."""
-        source_id = _inverted(self._sources).get(source)
-        if source_id:
+        if source_id := _inverted(self._sources).get(source):
             await self._tv.setSource(source_id)
         await self._async_update_soon()
 
@@ -211,9 +175,12 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
 
     async def async_turn_off(self):
         """Turn off the device."""
-        await self._tv.sendKey("Standby")
-        self._state = STATE_OFF
-        await self._async_update_soon()
+        if self._state == STATE_ON:
+            await self._tv.sendKey("Standby")
+            self._state = STATE_OFF
+            await self._async_update_soon()
+        else:
+            _LOGGER.debug("Ignoring turn off when already in expected state")
 
     async def async_volume_up(self):
         """Send volume up command."""
@@ -295,8 +262,8 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
     def media_image_url(self):
         """Image url of current playing media."""
         if self._media_content_id and self._media_content_type in (
-            MEDIA_CLASS_APP,
-            MEDIA_CLASS_CHANNEL,
+            MEDIA_TYPE_APP,
+            MEDIA_TYPE_CHANNEL,
         ):
             return self.get_browse_image_url(
                 self._media_content_type, self._media_content_id, media_image_id=None
@@ -311,33 +278,9 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
     @property
     def app_name(self):
         """Name of the current running app."""
-        app = self._tv.applications.get(self._tv.application_id)
-        if app:
+        if app := self._tv.applications.get(self._tv.application_id):
             return app.get("label")
 
-    @property
-    def device_class(self):
-        """Return the device class."""
-        return DEVICE_CLASS_TV
-
-    @property
-    def unique_id(self):
-        """Return unique identifier if known."""
-        return self._unique_id
-
-    @property
-    def device_info(self):
-        """Return a device description for device registry."""
-        return {
-            "name": self._system["name"],
-            "identifiers": {
-                (DOMAIN, self._unique_id),
-            },
-            "model": self._system.get("model"),
-            "manufacturer": "Philips",
-            "sw_version": self._system.get("softwareversion"),
-        }
-
     async def async_play_media(self, media_type, media_id, **kwargs):
         """Play a piece of media."""
         _LOGGER.debug("Call play media type <%s>, Id <%s>", media_type, media_id)
@@ -350,8 +293,7 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
             else:
                 _LOGGER.error("Unable to find channel <%s>", media_id)
         elif media_type == MEDIA_TYPE_APP:
-            app = self._tv.applications.get(media_id)
-            if app:
+            if app := self._tv.applications.get(media_id):
                 await self._tv.setApplication(app["intent"])
                 await self._async_update_soon()
             else:
@@ -370,9 +312,6 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
                     media_content_type=MEDIA_TYPE_CHANNEL,
                     can_play=True,
                     can_expand=False,
-                    thumbnail=self.get_browse_image_url(
-                        MEDIA_TYPE_APP, channel_id, media_image_id=None
-                    ),
                 )
                 for channel_id, channel in self._tv.channels.items()
             ]
@@ -384,7 +323,7 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
             media_class=MEDIA_CLASS_DIRECTORY,
             media_content_id="channels",
             media_content_type=MEDIA_TYPE_CHANNELS,
-            children_media_class=MEDIA_TYPE_CHANNEL,
+            children_media_class=MEDIA_CLASS_CHANNEL,
             can_play=False,
             can_expand=True,
             children=children,
@@ -410,9 +349,6 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
                         media_content_type=MEDIA_TYPE_CHANNEL,
                         can_play=True,
                         can_expand=False,
-                        thumbnail=self.get_browse_image_url(
-                            MEDIA_TYPE_APP, channel, media_image_id=None
-                        ),
                     )
                     for channel in favorites
                 ]
@@ -427,7 +363,7 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
             media_class=MEDIA_CLASS_DIRECTORY,
             media_content_id=f"favorites/{list_id}",
             media_content_type=MEDIA_TYPE_CHANNELS,
-            children_media_class=MEDIA_TYPE_CHANNEL,
+            children_media_class=MEDIA_CLASS_CHANNEL,
             can_play=False,
             can_expand=True,
             children=children,
@@ -458,7 +394,7 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
             media_class=MEDIA_CLASS_DIRECTORY,
             media_content_id="applications",
             media_content_type=MEDIA_TYPE_APPS,
-            children_media_class=MEDIA_TYPE_APP,
+            children_media_class=MEDIA_CLASS_APP,
             can_play=False,
             can_expand=True,
             children=children,
@@ -479,7 +415,7 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
             media_class=MEDIA_CLASS_DIRECTORY,
             media_content_id="favorite_lists",
             media_content_type=MEDIA_TYPE_CHANNELS,
-            children_media_class=MEDIA_TYPE_CHANNEL,
+            children_media_class=MEDIA_CLASS_CHANNEL,
             can_play=False,
             can_expand=True,
             children=children,
@@ -522,8 +458,11 @@ class PhilipsTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
         raise BrowseError(f"Media not found: {media_content_type} / {media_content_id}")
 
     async def async_get_browse_image(
-        self, media_content_type, media_content_id, media_image_id=None
-    ):
+        self,
+        media_content_type: str,
+        media_content_id: str,
+        media_image_id: str | None = None,
+    ) -> tuple[bytes | None, str | None]:
         """Serve album art. Returns (content, content_type)."""
         try:
             if media_content_type == MEDIA_TYPE_APP and media_content_id:
