@@ -20,16 +20,15 @@ from homeassistant.components.climate.const import (
     SUPPORT_PRESET_MODE,
     SUPPORT_TARGET_TEMPERATURE,
 )
-from homeassistant.components.shelly import BlockDeviceWrapper
-from homeassistant.components.shelly.utils import get_device_entry_gen
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import ATTR_TEMPERATURE, TEMP_CELSIUS
 from homeassistant.core import HomeAssistant, State, callback
-from homeassistant.helpers import device_registry, entity, entity_registry
+from homeassistant.helpers import device_registry, entity_registry, update_coordinator
 from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.restore_state import RestoreEntity
 
+from . import BlockDeviceWrapper
 from .const import (
     AIOSHELLY_DEVICE_TIMEOUT_SEC,
     BLOCK,
@@ -37,6 +36,7 @@ from .const import (
     DOMAIN,
     SHTRV_01_TEMPERATURE_SETTINGS,
 )
+from .utils import get_device_entry_gen
 
 _LOGGER: Final = logging.getLogger(__name__)
 
@@ -106,12 +106,13 @@ async def async_restore_climate_entities(
         _LOGGER.debug("Setup sleeping climate device %s", wrapper.name)
         _LOGGER.debug("Found entry %s [%s]", entry.original_name, entry.domain)
         async_add_entities([BlockSleepingClimate(wrapper, None, None, entry)])
+        break
 
 
 class BlockSleepingClimate(
+    update_coordinator.CoordinatorEntity,
     RestoreEntity,
     ClimateEntity,
-    entity.Entity,
 ):
     """Representation of a Shelly climate device."""
 
@@ -123,7 +124,6 @@ class BlockSleepingClimate(
     _attr_target_temperature_step = SHTRV_01_TEMPERATURE_SETTINGS["step"]
     _attr_temperature_unit = TEMP_CELSIUS
 
-    # pylint: disable=super-init-not-called
     def __init__(
         self,
         wrapper: BlockDeviceWrapper,
@@ -133,6 +133,8 @@ class BlockSleepingClimate(
     ) -> None:
         """Initialize climate."""
 
+        super().__init__(wrapper)
+
         self.wrapper = wrapper
         self.block: Block | None = sensor_block
         self.control_result: dict[str, Any] | None = None
@@ -153,6 +155,8 @@ class BlockSleepingClimate(
         elif entry is not None:
             self._unique_id = entry.unique_id
 
+        self._channel = cast(int, self._unique_id.split("_")[1])
+
     @property
     def unique_id(self) -> str:
         """Set unique id of entity."""
@@ -163,11 +167,6 @@ class BlockSleepingClimate(
         """Name of entity."""
         return self.wrapper.name
 
-    @property
-    def should_poll(self) -> bool:
-        """If device should be polled."""
-        return False
-
     @property
     def target_temperature(self) -> float | None:
         """Set target temperature."""
@@ -219,7 +218,7 @@ class BlockSleepingClimate(
             return CURRENT_HVAC_OFF
 
         return (
-            CURRENT_HVAC_IDLE if self.device_block.status == "0" else CURRENT_HVAC_HEAT
+            CURRENT_HVAC_HEAT if bool(self.device_block.status) else CURRENT_HVAC_IDLE
         )
 
     @property
@@ -234,13 +233,6 @@ class BlockSleepingClimate(
             "connections": {(device_registry.CONNECTION_NETWORK_MAC, self.wrapper.mac)}
         }
 
-    @property
-    def channel(self) -> str | None:
-        """Device channel."""
-        if self.block is not None:
-            return self.block.channel
-        return self.last_state_attributes.get("channel")
-
     def _check_is_off(self) -> bool:
         """Return if valve is off or on."""
         return bool(
@@ -254,7 +246,7 @@ class BlockSleepingClimate(
         try:
             async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
                 return await self.wrapper.device.http_request(
-                    "get", f"thermostat/{self.channel}", kwargs
+                    "get", f"thermostat/{self._channel}", kwargs
                 )
         except (asyncio.TimeoutError, OSError) as err:
             _LOGGER.error(
@@ -281,10 +273,10 @@ class BlockSleepingClimate(
 
     async def async_set_preset_mode(self, preset_mode: str) -> None:
         """Set preset mode."""
-        if not self._attr_preset_modes:
+        if not self._preset_modes:
             return
 
-        preset_index = self._attr_preset_modes.index(preset_mode)
+        preset_index = self._preset_modes.index(preset_mode)
 
         if preset_index == 0:
             await self.set_state_full_path(schedule=0)
@@ -306,14 +298,10 @@ class BlockSleepingClimate(
                 list, self.last_state.attributes.get("preset_modes")
             )
 
-        self.async_on_remove(self.wrapper.async_add_listener(self._update_callback))
-
-    async def async_update(self) -> None:
-        """Update entity with latest info."""
-        await self.wrapper.async_request_refresh()
+        await super().async_added_to_hass()
 
     @callback
-    def _update_callback(self) -> None:
+    def _handle_coordinator_update(self) -> None:
         """Handle device update."""
         if not self.wrapper.device.initialized:
             self.async_write_ha_state()
@@ -327,6 +315,6 @@ class BlockSleepingClimate(
             if hasattr(block, "targetTemp"):
                 self.block = block
 
-                _LOGGER.debug("Entity %s attached to block", self.name)
-                self.async_write_ha_state()
-                return
+        if self.device_block and self.block:
+            _LOGGER.debug("Entity %s attached to blocks", self.name)
+            self.async_write_ha_state()
