@@ -12,11 +12,13 @@ from typing import Any
 import urllib.parse
 
 import async_timeout
+import defusedxml.ElementTree as ET
 from soco.core import MUSIC_SRC_LINE_IN, MUSIC_SRC_RADIO, MUSIC_SRC_TV, SoCo
 from soco.data_structures import DidlAudioBroadcast, DidlPlaylistContainer
 from soco.events_base import Event as SonosEvent, SubscriptionBase
 from soco.exceptions import SoCoException, SoCoSlaveException, SoCoUPnPException
 from soco.music_library import MusicLibrary
+from soco.plugins.plex import PlexPlugin
 from soco.plugins.sharelink import ShareLinkPlugin
 from soco.snapshot import Snapshot
 
@@ -46,6 +48,7 @@ from .const import (
     SONOS_CREATE_BATTERY,
     SONOS_CREATE_LEVELS,
     SONOS_CREATE_MEDIA_PLAYER,
+    SONOS_CREATE_MIC_SENSOR,
     SONOS_CREATE_SWITCHES,
     SONOS_POLL_UPDATE,
     SONOS_REBOOTED,
@@ -54,12 +57,14 @@ from .const import (
     SONOS_STATE_PLAYING,
     SONOS_STATE_TRANSITIONING,
     SONOS_STATE_UPDATED,
+    SONOS_VANISHED,
     SOURCE_LINEIN,
     SOURCE_TV,
     SUBSCRIPTION_TIMEOUT,
 )
 from .favorites import SonosFavorites
 from .helpers import soco_error
+from .statistics import ActivityStatistics, EventStatistics
 
 NEVER_TIME = -1200.0
 EVENT_CHARGING = {
@@ -153,15 +158,29 @@ class SonosSpeaker:
         self.soco = soco
         self.household_id: str = soco.household_id
         self.media = SonosMedia(soco)
+        self._plex_plugin: PlexPlugin | None = None
         self._share_link_plugin: ShareLinkPlugin | None = None
         self.available = True
 
+        # Device information
+        self.hardware_version = speaker_info["hardware_version"]
+        self.software_version = speaker_info["software_version"]
+        self.mac_address = speaker_info["mac_address"]
+        self.model_name = speaker_info["model_name"]
+        self.model_number = speaker_info["model_number"]
+        self.uid = speaker_info["uid"]
+        self.version = speaker_info["display_version"]
+        self.zone_name = speaker_info["zone_name"]
+
         # Subscriptions and events
         self.subscriptions_failed: bool = False
         self._subscriptions: list[SubscriptionBase] = []
         self._resubscription_lock: asyncio.Lock | None = None
         self._event_dispatchers: dict[str, Callable] = {}
         self._last_activity: float = NEVER_TIME
+        self._last_event_cache: dict[str, Any] = {}
+        self.activity_stats: ActivityStatistics = ActivityStatistics(self.zone_name)
+        self.event_stats: EventStatistics = EventStatistics(self.zone_name)
 
         # Scheduled callback handles
         self._poll_timer: Callable | None = None
@@ -169,13 +188,6 @@ class SonosSpeaker:
         # Dispatcher handles
         self.dispatchers: list[Callable] = []
 
-        # Device information
-        self.mac_address = speaker_info["mac_address"]
-        self.model_name = speaker_info["model_name"]
-        self.uid = speaker_info["uid"]
-        self.version = speaker_info["display_version"]
-        self.zone_name = speaker_info["zone_name"]
-
         # Battery
         self.battery_info: dict[str, Any] = {}
         self._last_battery_event: datetime.datetime | None = None
@@ -184,16 +196,20 @@ class SonosSpeaker:
         # Volume / Sound
         self.volume: int | None = None
         self.muted: bool | None = None
-        self.night_mode: bool | None = None
-        self.dialog_level: bool | None = None
         self.cross_fade: bool | None = None
         self.bass: int | None = None
         self.treble: int | None = None
+
+        # Home theater
+        self.audio_delay: int | None = None
+        self.dialog_level: bool | None = None
+        self.night_mode: bool | None = None
         self.sub_enabled: bool | None = None
         self.surround_enabled: bool | None = None
 
         # Misc features
         self.buttons_enabled: bool | None = None
+        self.mic_enabled: bool | None = None
         self.status_light: bool | None = None
 
         # Grouping
@@ -211,6 +227,7 @@ class SonosSpeaker:
             (SONOS_SPEAKER_ADDED, self.update_group_for_uid),
             (f"{SONOS_REBOOTED}-{self.soco.uid}", self.async_rebooted),
             (f"{SONOS_SPEAKER_ACTIVITY}-{self.soco.uid}", self.speaker_activity),
+            (f"{SONOS_VANISHED}-{self.soco.uid}", self.async_vanished),
         )
 
         for (signal, target) in dispatch_pairs:
@@ -301,6 +318,13 @@ class SonosSpeaker:
         """Return true if player is a coordinator."""
         return self.coordinator is None
 
+    @property
+    def plex_plugin(self) -> PlexPlugin:
+        """Cache the PlexPlugin instance for this speaker."""
+        if not self._plex_plugin:
+            self._plex_plugin = PlexPlugin(self.soco)
+        return self._plex_plugin
+
     @property
     def share_link(self) -> ShareLinkPlugin:
         """Cache the ShareLinkPlugin instance for this speaker."""
@@ -367,6 +391,8 @@ class SonosSpeaker:
 
     async def async_unsubscribe(self) -> None:
         """Cancel all subscriptions."""
+        if not self._subscriptions:
+            return
         _LOGGER.debug("Unsubscribing from events for %s", self.zone_name)
         results = await asyncio.gather(
             *(subscription.unsubscribe() for subscription in self._subscriptions),
@@ -417,49 +443,53 @@ class SonosSpeaker:
             self._poll_timer = None
 
         self.speaker_activity(f"{event.service.service_type} subscription")
+        self.event_stats.receive(event)
 
+        # Skip if this update is an unchanged subset of the previous event
+        if last_event := self._last_event_cache.get(event.service.service_type):
+            if event.variables.items() <= last_event.items():
+                self.event_stats.duplicate(event)
+                return
+
+        # Save most recently processed event variables for cache and diagnostics
+        self._last_event_cache[event.service.service_type] = event.variables
         dispatcher = self._event_dispatchers[event.service.service_type]
         dispatcher(event)
 
     @callback
     def async_dispatch_alarms(self, event: SonosEvent) -> None:
         """Add the soco instance associated with the event to the callback."""
-        if not (event_id := event.variables.get("alarm_list_version")):
+        if "alarm_list_version" not in event.variables:
             return
-        self.alarms.async_handle_event(event_id, self.soco)
+        self.hass.async_create_task(self.alarms.async_process_event(event, self))
 
     @callback
     def async_dispatch_device_properties(self, event: SonosEvent) -> None:
         """Update device properties from an event."""
+        self.event_stats.process(event)
         self.hass.async_create_task(self.async_update_device_properties(event))
 
     async def async_update_device_properties(self, event: SonosEvent) -> None:
         """Update device properties from an event."""
+        if "mic_enabled" in event.variables:
+            mic_exists = self.mic_enabled is not None
+            self.mic_enabled = bool(int(event.variables["mic_enabled"]))
+            if not mic_exists:
+                async_dispatcher_send(self.hass, SONOS_CREATE_MIC_SENSOR, self)
+
         if more_info := event.variables.get("more_info"):
-            battery_dict = dict(x.split(":") for x in more_info.split(","))
-            for unused in UNUSED_DEVICE_KEYS:
-                battery_dict.pop(unused, None)
-            if not battery_dict:
-                return
-            if "BattChg" not in battery_dict:
-                _LOGGER.debug(
-                    "Unknown device properties update for %s (%s), please report an issue: '%s'",
-                    self.zone_name,
-                    self.model_name,
-                    more_info,
-                )
-                return
-            await self.async_update_battery_info(battery_dict)
+            await self.async_update_battery_info(more_info)
+
         self.async_write_entity_states()
 
     @callback
     def async_dispatch_favorites(self, event: SonosEvent) -> None:
         """Add the soco instance associated with the event to the callback."""
-        if not (event_id := event.variables.get("favorites_update_id")):
+        if "favorites_update_id" not in event.variables:
             return
-        if not (container_ids := event.variables.get("container_update_i_ds")):
+        if "container_update_i_ds" not in event.variables:
             return
-        self.favorites.async_handle_event(event_id, container_ids, self.soco)
+        self.hass.async_create_task(self.favorites.async_process_event(event, self))
 
     @callback
     def async_dispatch_media_update(self, event: SonosEvent) -> None:
@@ -472,6 +502,7 @@ class SonosSpeaker:
     @callback
     def async_update_volume(self, event: SonosEvent) -> None:
         """Update information about currently volume settings."""
+        self.event_stats.process(event)
         variables = event.variables
 
         if "volume" in variables:
@@ -489,7 +520,7 @@ class SonosSpeaker:
             if bool_var in variables:
                 setattr(self, bool_var, variables[bool_var] == "1")
 
-        for int_var in ("bass", "treble"):
+        for int_var in ("audio_delay", "bass", "treble"):
             if int_var in variables:
                 setattr(self, int_var, variables[int_var])
 
@@ -503,6 +534,7 @@ class SonosSpeaker:
         """Track the last activity on this speaker, set availability and resubscribe."""
         _LOGGER.debug("Activity on %s from %s", self.zone_name, source)
         self._last_activity = time.monotonic()
+        self.activity_stats.activity(source, self._last_activity)
         was_available = self.available
         self.available = True
         if not was_available:
@@ -545,6 +577,15 @@ class SonosSpeaker:
         self.hass.data[DATA_SONOS].discovery_known.discard(self.soco.uid)
         self.async_write_entity_states()
 
+    async def async_vanished(self, reason: str) -> None:
+        """Handle removal of speaker when marked as vanished."""
+        if not self.available:
+            return
+        _LOGGER.debug(
+            "%s has vanished (%s), marking unavailable", self.zone_name, reason
+        )
+        await self.async_offline()
+
     async def async_rebooted(self, soco: SoCo) -> None:
         """Handle a detected speaker reboot."""
         _LOGGER.warning(
@@ -559,8 +600,22 @@ class SonosSpeaker:
     #
     # Battery management
     #
-    async def async_update_battery_info(self, battery_dict: dict[str, Any]) -> None:
-        """Update battery info using the decoded SonosEvent."""
+    async def async_update_battery_info(self, more_info: str) -> None:
+        """Update battery info using a SonosEvent payload value."""
+        battery_dict = dict(x.split(":") for x in more_info.split(","))
+        for unused in UNUSED_DEVICE_KEYS:
+            battery_dict.pop(unused, None)
+        if not battery_dict:
+            return
+        if "BattChg" not in battery_dict:
+            _LOGGER.debug(
+                "Unknown device properties update for %s (%s), please report an issue: '%s'",
+                self.zone_name,
+                self.model_name,
+                more_info,
+            )
+            return
+
         self._last_battery_event = dt_util.utcnow()
 
         is_charging = EVENT_CHARGING[battery_dict["BattChg"]]
@@ -644,8 +699,27 @@ class SonosSpeaker:
     @callback
     def async_update_groups(self, event: SonosEvent) -> None:
         """Handle callback for topology change event."""
-        if not hasattr(event, "zone_player_uui_ds_in_group"):
+        if xml := event.variables.get("zone_group_state"):
+            zgs = ET.fromstring(xml)
+            for vanished_device in zgs.find("VanishedDevices"):
+                if (reason := vanished_device.get("Reason")) != "sleeping":
+                    _LOGGER.debug(
+                        "Ignoring %s marked %s as vanished with reason: %s",
+                        self.zone_name,
+                        vanished_device.get("ZoneName"),
+                        reason,
+                    )
+                    continue
+                uid = vanished_device.get("UUID")
+                async_dispatcher_send(
+                    self.hass,
+                    f"{SONOS_VANISHED}-{uid}",
+                    reason,
+                )
+
+        if "zone_player_uui_ds_in_group" not in event.variables:
             return
+        self.event_stats.process(event)
         self.hass.async_create_task(self.create_update_groups_coro(event))
 
     def create_update_groups_coro(self, event: SonosEvent | None = None) -> Coroutine:
@@ -959,15 +1033,13 @@ class SonosSpeaker:
     #
     # Media and playback state handlers
     #
-    @soco_error()
+    @soco_error(raise_on_err=False)
     def update_volume(self) -> None:
         """Update information about current volume settings."""
         self.volume = self.soco.volume
         self.muted = self.soco.mute
         self.night_mode = self.soco.night_mode
         self.dialog_level = self.soco.dialog_level
-        self.bass = self.soco.bass
-        self.treble = self.soco.treble
 
         try:
             self.cross_fade = self.soco.cross_fade
@@ -991,6 +1063,9 @@ class SonosSpeaker:
         if new_status == SONOS_STATE_TRANSITIONING:
             return
 
+        if event:
+            self.event_stats.process(event)
+
         self.media.clear()
         update_position = new_status != self.media.playback_status
         self.media.playback_status = new_status
