@@ -3,11 +3,11 @@ from __future__ import annotations
 
 from collections.abc import Callable
 from dataclasses import dataclass, field
-from enum import Enum
 import logging
 
 from tuya_iot import TuyaCloudOpenAPIEndpoint
 
+from homeassistant.backports.enum import StrEnum
 from homeassistant.components.sensor import SensorDeviceClass
 from homeassistant.const import (
     CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
@@ -36,6 +36,7 @@ from homeassistant.const import (
     TEMP_FAHRENHEIT,
     VOLUME_CUBIC_FEET,
     VOLUME_CUBIC_METERS,
+    Platform,
 )
 
 DOMAIN = "tuya"
@@ -51,21 +52,6 @@ CONF_PASSWORD = "password"
 CONF_COUNTRY_CODE = "country_code"
 CONF_APP_TYPE = "tuya_app_type"
 
-DEVICE_CLASS_TUYA_BASIC_ANTI_FLICKR = "tuya__basic_anti_flickr"
-DEVICE_CLASS_TUYA_BASIC_NIGHTVISION = "tuya__basic_nightvision"
-DEVICE_CLASS_TUYA_DECIBEL_SENSITIVITY = "tuya__decibel_sensitivity"
-DEVICE_CLASS_TUYA_IPC_WORK_MODE = "tuya__ipc_work_mode"
-DEVICE_CLASS_TUYA_LED_TYPE = "tuya__led_type"
-DEVICE_CLASS_TUYA_LIGHT_MODE = "tuya__light_mode"
-DEVICE_CLASS_TUYA_MOTION_SENSITIVITY = "tuya__motion_sensitivity"
-DEVICE_CLASS_TUYA_RECORD_MODE = "tuya__record_mode"
-DEVICE_CLASS_TUYA_RELAY_STATUS = "tuya__relay_status"
-DEVICE_CLASS_TUYA_STATUS = "tuya__status"
-DEVICE_CLASS_TUYA_FINGERBOT_MODE = "tuya__fingerbot_mode"
-DEVICE_CLASS_TUYA_VACUUM_CISTERN = "tuya__vacuum_cistern"
-DEVICE_CLASS_TUYA_VACUUM_COLLECTION = "tuya__vacuum_collection"
-DEVICE_CLASS_TUYA_VACUUM_MODE = "tuya__vacuum_mode"
-
 TUYA_DISCOVERY_NEW = "tuya_discovery_new"
 TUYA_HA_SIGNAL_UPDATE_ENTITY = "tuya_entry_update"
 
@@ -79,25 +65,46 @@ TUYA_SMART_APP = "tuyaSmart"
 SMARTLIFE_APP = "smartlife"
 
 PLATFORMS = [
-    "binary_sensor",
-    "button",
-    "camera",
-    "climate",
-    "cover",
-    "fan",
-    "humidifier",
-    "light",
-    "number",
-    "scene",
-    "select",
-    "sensor",
-    "siren",
-    "switch",
-    "vacuum",
+    Platform.ALARM_CONTROL_PANEL,
+    Platform.BINARY_SENSOR,
+    Platform.BUTTON,
+    Platform.CAMERA,
+    Platform.CLIMATE,
+    Platform.COVER,
+    Platform.FAN,
+    Platform.HUMIDIFIER,
+    Platform.LIGHT,
+    Platform.NUMBER,
+    Platform.SCENE,
+    Platform.SELECT,
+    Platform.SENSOR,
+    Platform.SIREN,
+    Platform.SWITCH,
+    Platform.VACUUM,
 ]
 
 
-class WorkMode(str, Enum):
+class TuyaDeviceClass(StrEnum):
+    """Tuya specific device classes, used for translations."""
+
+    BASIC_ANTI_FLICKR = "tuya__basic_anti_flickr"
+    BASIC_NIGHTVISION = "tuya__basic_nightvision"
+    DECIBEL_SENSITIVITY = "tuya__decibel_sensitivity"
+    FAN_ANGLE = "tuya__fan_angle"
+    FINGERBOT_MODE = "tuya__fingerbot_mode"
+    IPC_WORK_MODE = "tuya__ipc_work_mode"
+    LED_TYPE = "tuya__led_type"
+    LIGHT_MODE = "tuya__light_mode"
+    MOTION_SENSITIVITY = "tuya__motion_sensitivity"
+    RECORD_MODE = "tuya__record_mode"
+    RELAY_STATUS = "tuya__relay_status"
+    STATUS = "tuya__status"
+    VACUUM_CISTERN = "tuya__vacuum_cistern"
+    VACUUM_COLLECTION = "tuya__vacuum_collection"
+    VACUUM_MODE = "tuya__vacuum_mode"
+
+
+class WorkMode(StrEnum):
     """Work modes."""
 
     COLOUR = "colour"
@@ -106,8 +113,19 @@ class WorkMode(str, Enum):
     WHITE = "white"
 
 
-class DPCode(str, Enum):
-    """Device Property Codes used by Tuya.
+class DPType(StrEnum):
+    """Data point types."""
+
+    BOOLEAN = "Boolean"
+    ENUM = "Enum"
+    INTEGER = "Integer"
+    JSON = "Json"
+    RAW = "Raw"
+    STRING = "String"
+
+
+class DPCode(StrEnum):
+    """Data Point Codes used by Tuya.
 
     https://developer.tuya.com/en/docs/iot/standarddescription?id=K9i5ql6waswzq
     """
@@ -115,6 +133,7 @@ class DPCode(str, Enum):
     ALARM_SWITCH = "alarm_switch"  # Alarm switch
     ALARM_TIME = "alarm_time"  # Alarm time
     ALARM_VOLUME = "alarm_volume"  # Alarm volume
+    ALARM_MESSAGE = "alarm_message"
     ANGLE_HORIZONTAL = "angle_horizontal"
     ANGLE_VERTICAL = "angle_vertical"
     ANION = "anion"  # Ionizer unit
@@ -168,6 +187,8 @@ class DPCode(str, Enum):
     CONTROL = "control"
     CONTROL_2 = "control_2"
     CONTROL_3 = "control_3"
+    COUNTDOWN = "countdown"  # Countdown
+    COUNTDOWN_SET = "countdown_set"  # Countdown setting
     CRY_DETECTION_SWITCH = "cry_detection_switch"
     CUP_NUMBER = "cup_number"  # NUmber of cups
     CUR_CURRENT = "cur_current"  # Actual current
@@ -183,11 +204,19 @@ class DPCode(str, Enum):
     DUSTER_CLOTH = "duster_cloth"
     EDGE_BRUSH = "edge_brush"
     ELECTRICITY_LEFT = "electricity_left"
+    FAN_BEEP = "fan_beep"  # Sound
+    FAN_COOL = "fan_cool"  # Cool wind
     FAN_DIRECTION = "fan_direction"  # Fan direction
+    FAN_HORIZONTAL = "fan_horizontal"  # Horizontal swing flap angle
+    FAN_SPEED = "fan_speed"
     FAN_SPEED_ENUM = "fan_speed_enum"  # Speed mode
     FAN_SPEED_PERCENT = "fan_speed_percent"  # Stepless speed
+    FAN_MODE = "fan_mode"
+    FAN_VERTICAL = "fan_vertical"  # Vertical swing flap angle
     FAR_DETECTION = "far_detection"
     FAULT = "fault"
+    FEED_REPORT = "feed_report"
+    FEED_STATE = "feed_state"
     FILTER_LIFE = "filter"
     FILTER_RESET = "filter_reset"  # Filter (cartridge) reset
     FLOODLIGHT_LIGHTNESS = "floodlight_lightness"
@@ -196,6 +225,7 @@ class DPCode(str, Enum):
     GAS_SENSOR_STATE = "gas_sensor_state"
     GAS_SENSOR_STATUS = "gas_sensor_status"
     GAS_SENSOR_VALUE = "gas_sensor_value"
+    HUMIDIFIER = "humidifier"  # Humidification
     HUMIDITY_CURRENT = "humidity_current"  # Current humidity
     HUMIDITY_SET = "humidity_set"  # Humidity setting
     HUMIDITY_VALUE = "humidity_value"  # Humidity
@@ -207,12 +237,16 @@ class DPCode(str, Enum):
     LIGHT = "light"  # Light
     LIGHT_MODE = "light_mode"
     LOCK = "lock"  # Lock / Child lock
+    MASTER_MODE = "master_mode"  # alarm mode
+    MACH_OPERATE = "mach_operate"
+    MANUAL_FEED = "manual_feed"
     MATERIAL = "material"  # Material
     MODE = "mode"  # Working mode / Mode
     MOTION_RECORD = "motion_record"
     MOTION_SENSITIVITY = "motion_sensitivity"
     MOTION_SWITCH = "motion_switch"  # Motion switch
     MOTION_TRACKING = "motion_tracking"
+    MOVEMENT_DETECT_PIC = "movement_detect_pic"
     MUFFLING = "muffling"  # Muffling
     NEAR_DETECTION = "near_detection"
     PAUSE = "pause"
@@ -222,6 +256,7 @@ class DPCode(str, Enum):
     PERCENT_STATE = "percent_state"
     PERCENT_STATE_2 = "percent_state_2"
     PERCENT_STATE_3 = "percent_state_3"
+    POSITION = "position"
     PHASE_A = "phase_a"
     PHASE_B = "phase_b"
     PHASE_C = "phase_c"
@@ -237,6 +272,7 @@ class DPCode(str, Enum):
     PRESSURE_STATE = "pressure_state"
     PRESSURE_VALUE = "pressure_value"
     PUMP_RESET = "pump_reset"  # Water pump reset
+    OXYGEN = "oxygen"  # Oxygen bar
     RECORD_MODE = "record_mode"
     RECORD_SWITCH = "record_switch"  # Recording switch
     RELAY_STATUS = "relay_status"
@@ -254,6 +290,7 @@ class DPCode(str, Enum):
     SHOCK_STATE = "shock_state"  # Vibration status
     SIREN_SWITCH = "siren_switch"
     SITUATION_SET = "situation_set"
+    SLOW_FEED = "slow_feed"
     SMOKE_SENSOR_STATE = "smoke_sensor_state"
     SMOKE_SENSOR_STATUS = "smoke_sensor_status"
     SMOKE_SENSOR_VALUE = "smoke_sensor_value"
@@ -275,6 +312,7 @@ class DPCode(str, Enum):
     SWITCH_CHARGE = "switch_charge"
     SWITCH_CONTROLLER = "switch_controller"
     SWITCH_DISTURB = "switch_disturb"
+    SWITCH_FAN = "switch_fan"
     SWITCH_HORIZONTAL = "switch_horizontal"  # Horizontal swing flap switch
     SWITCH_LED = "switch_led"  # Switch
     SWITCH_LED_1 = "switch_led_1"
@@ -291,6 +329,7 @@ class DPCode(str, Enum):
     SWITCH_USB6 = "switch_usb6"  # USB 6
     SWITCH_VERTICAL = "switch_vertical"  # Vertical swing flap switch
     SWITCH_VOICE = "switch_voice"  # Voice switch
+    TEMP = "temp"  # Temperature setting
     TEMP_BOILING_C = "temp_boiling_c"
     TEMP_BOILING_F = "temp_boiling_f"
     TEMP_CONTROLLER = "temp_controller"
@@ -305,6 +344,7 @@ class DPCode(str, Enum):
     TOTAL_CLEAN_AREA = "total_clean_area"
     TOTAL_CLEAN_COUNT = "total_clean_count"
     TOTAL_CLEAN_TIME = "total_clean_time"
+    TOTAL_FORWARD_ENERGY = "total_forward_energy"
     UV = "uv"  # UV sterilization
     VA_BATTERY = "va_battery"
     VA_HUMIDITY = "va_humidity"
@@ -312,6 +352,7 @@ class DPCode(str, Enum):
     VOC_STATE = "voc_state"
     VOC_VALUE = "voc_value"
     VOICE_SWITCH = "voice_switch"
+    VOICE_TIMES = "voice_times"
     VOLUME_SET = "volume_set"
     WARM = "warm"  # Heat preservation
     WARM_TIME = "warm_time"  # Heat preservation time
