@@ -2,12 +2,14 @@
 import logging
 
 from homeassistant.components.binary_sensor import (
-    DEVICE_CLASS_MOISTURE,
-    DEVICE_CLASS_OPENING,
+    BinarySensorDeviceClass,
     BinarySensorEntity,
 )
-from homeassistant.core import callback
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.event import async_call_later
+from homeassistant.helpers.restore_state import RestoreEntity
 
 from . import XiaomiDevice
 from .const import DOMAIN, GATEWAYS_KEY
@@ -26,29 +28,33 @@ DENSITY = "density"
 ATTR_DENSITY = "Density"
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Perform the setup for Xiaomi devices."""
     entities = []
     gateway = hass.data[DOMAIN][GATEWAYS_KEY][config_entry.entry_id]
     for entity in gateway.devices["binary_sensor"]:
         model = entity["model"]
-        if model in ["motion", "sensor_motion", "sensor_motion.aq2"]:
+        if model in ("motion", "sensor_motion", "sensor_motion.aq2"):
             entities.append(XiaomiMotionSensor(entity, hass, gateway, config_entry))
-        elif model in ["magnet", "sensor_magnet", "sensor_magnet.aq2"]:
+        elif model in ("magnet", "sensor_magnet", "sensor_magnet.aq2"):
             entities.append(XiaomiDoorSensor(entity, gateway, config_entry))
         elif model == "sensor_wleak.aq1":
             entities.append(XiaomiWaterLeakSensor(entity, gateway, config_entry))
-        elif model in ["smoke", "sensor_smoke"]:
+        elif model in ("smoke", "sensor_smoke"):
             entities.append(XiaomiSmokeSensor(entity, gateway, config_entry))
-        elif model in ["natgas", "sensor_natgas"]:
+        elif model in ("natgas", "sensor_natgas"):
             entities.append(XiaomiNatgasSensor(entity, gateway, config_entry))
-        elif model in [
+        elif model in (
             "switch",
             "sensor_switch",
             "sensor_switch.aq2",
             "sensor_switch.aq3",
             "remote.b1acn01",
-        ]:
+        ):
             if "proto" not in entity or int(entity["proto"][0:1]) == 1:
                 data_key = "status"
             else:
@@ -56,13 +62,13 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
             entities.append(
                 XiaomiButton(entity, "Switch", data_key, hass, gateway, config_entry)
             )
-        elif model in [
+        elif model in (
             "86sw1",
             "sensor_86sw1",
             "sensor_86sw1.aq1",
             "remote.b186acn01",
             "remote.b186acn02",
-        ]:
+        ):
             if "proto" not in entity or int(entity["proto"][0:1]) == 1:
                 data_key = "channel_0"
             else:
@@ -72,13 +78,13 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
                     entity, "Wall Switch", data_key, hass, gateway, config_entry
                 )
             )
-        elif model in [
+        elif model in (
             "86sw2",
             "sensor_86sw2",
             "sensor_86sw2.aq1",
             "remote.b286acn01",
             "remote.b286acn02",
-        ]:
+        ):
             if "proto" not in entity or int(entity["proto"][0:1]) == 1:
                 data_key_left = "channel_0"
                 data_key_right = "channel_1"
@@ -115,9 +121,9 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
                     config_entry,
                 )
             )
-        elif model in ["cube", "sensor_cube", "sensor_cube.aqgl01"]:
+        elif model in ("cube", "sensor_cube", "sensor_cube.aqgl01"):
             entities.append(XiaomiCube(entity, hass, gateway, config_entry))
-        elif model in ["vibration", "vibration.aq1"]:
+        elif model in ("vibration", "vibration.aq1"):
             entities.append(
                 XiaomiVibration(entity, "Vibration", "status", gateway, config_entry)
             )
@@ -176,6 +182,11 @@ class XiaomiNatgasSensor(XiaomiBinarySensor):
         attrs.update(super().extra_state_attributes)
         return attrs
 
+    async def async_added_to_hass(self) -> None:
+        """Handle entity which will be added."""
+        await super().async_added_to_hass()
+        self._state = False
+
     def parse_data(self, data, raw_data):
         """Parse data sent by gateway."""
         if DENSITY in data:
@@ -227,6 +238,11 @@ class XiaomiMotionSensor(XiaomiBinarySensor):
         self._state = False
         self.async_write_ha_state()
 
+    async def async_added_to_hass(self) -> None:
+        """Handle entity which will be added."""
+        await super().async_added_to_hass()
+        self._state = False
+
     def parse_data(self, data, raw_data):
         """Parse data sent by gateway.
 
@@ -288,7 +304,7 @@ class XiaomiMotionSensor(XiaomiBinarySensor):
             return True
 
 
-class XiaomiDoorSensor(XiaomiBinarySensor):
+class XiaomiDoorSensor(XiaomiBinarySensor, RestoreEntity):
     """Representation of a XiaomiDoorSensor."""
 
     def __init__(self, device, xiaomi_hub, config_entry):
@@ -303,7 +319,7 @@ class XiaomiDoorSensor(XiaomiBinarySensor):
             "Door Window Sensor",
             xiaomi_hub,
             data_key,
-            DEVICE_CLASS_OPENING,
+            BinarySensorDeviceClass.OPENING,
             config_entry,
         )
 
@@ -314,6 +330,15 @@ class XiaomiDoorSensor(XiaomiBinarySensor):
         attrs.update(super().extra_state_attributes)
         return attrs
 
+    async def async_added_to_hass(self) -> None:
+        """Handle entity which will be added."""
+        await super().async_added_to_hass()
+        state = await self.async_get_last_state()
+        if state is None:
+            return
+
+        self._state = state.state == "on"
+
     def parse_data(self, data, raw_data):
         """Parse data sent by gateway."""
         self._should_poll = False
@@ -353,10 +378,15 @@ class XiaomiWaterLeakSensor(XiaomiBinarySensor):
             "Water Leak Sensor",
             xiaomi_hub,
             data_key,
-            DEVICE_CLASS_MOISTURE,
+            BinarySensorDeviceClass.MOISTURE,
             config_entry,
         )
 
+    async def async_added_to_hass(self) -> None:
+        """Handle entity which will be added."""
+        await super().async_added_to_hass()
+        self._state = False
+
     def parse_data(self, data, raw_data):
         """Parse data sent by gateway."""
         self._should_poll = False
@@ -395,6 +425,11 @@ class XiaomiSmokeSensor(XiaomiBinarySensor):
         attrs.update(super().extra_state_attributes)
         return attrs
 
+    async def async_added_to_hass(self) -> None:
+        """Handle entity which will be added."""
+        await super().async_added_to_hass()
+        self._state = False
+
     def parse_data(self, data, raw_data):
         """Parse data sent by gateway."""
         if DENSITY in data:
